
from twisted.trial import unittest
from twisted.spread import pb
from twisted.internet import reactor, defer

import petmail.crypto
petmail.crypto.EncryptionAgent = petmail.crypto.FakeEncryptionAgent

from petmail import util, ticket, crypto
from petmail.server import ticketserver

class TestClient(ticketserver.PBTicketClient):
    def displayImage(self, image):
        self.sendResponse(image)

class Test(unittest.TestCase):
    def setUp(self):
        self.crypto = crypto.FakeEncryptionAgent()
        d = self.crypto.generateKey("ticketserver")
        d.addCallback(self._setUp_1)
        return d
    def _setUp_1(self, privkey):
        server = ticketserver.PBTicketServer(None, privkey, fake=True)
        factory = pb.PBServerFactory(server, unsafeTracebacks=True)
        p = self.port = reactor.listenTCP(0, factory)
        portnum = p.getHost().port
        self.url = "PB:localhost:%d" % portnum
        server.url = self.url
        
    def tearDown(self):
        # wait an extra bit so the server-side of the client connection will
        # be disconnected too.
        d = defer.Deferred()
        reactor.callLater(0.1, d.callback, None)
        dl = [d, defer.maybeDeferred(self.port.stopListening)]
        return defer.DeferredList(dl)

    def testGetKey(self):
        d = ticketserver.PBTicketKeyClient(self.url).getKey()
        d.addCallback(self._testGetKey_1)
        return d
    testGetKey.timeout = 60
    def _testGetKey_1(self, keydata):
        d = self.crypto.addPubkey(keydata)
        d.addCallback(self._testGetKey_2)
        return d
    def _testGetKey_2(self, key):
        return key

    def testGetTicket(self):
        d = self.testGetKey()
        d.addCallback(self._testGetTicket_1)
        return d
    testGetTicket.timeout = 60
    def _testGetTicket_1(self, key):
        now = util.now()
        d = TestClient(self.url).getTicket()
        d.addCallback(self._testGetTicket_2, key, now)
        return d
    def _testGetTicket_2(self, signed, key, now):
        d = self.crypto.decrypt(signed)
        d.addCallback(self._testGetTicket_3, key, now)
        return d
    def _testGetTicket_3(self, res, key, now):
        recip, sigID, plaintext = res
        t = util.decode(plaintext)
        self.failUnlessEqual(sigID, key.getID())
        self.failUnlessEqual(t['URL'], self.url)
        self.assertApproximates(now, t['Timestamp'], 5)
        self.failUnless(t['TicketID'])

        taker = ticket.TicketTaker(self.url, None, key)
        self.failUnless(taker.checkTicket(sigID, t))
